"""
@Author  ：duomei
@File    ：exception_utils.py
@Time    ：2025/3/11 13:38
"""

# 自定义异常类
class TableNameError(Exception):
    __doc__ = "数据库表名命名规则验证错误"


class ColumnNameError(Exception):
    __doc__ = "数据库表属性命名规则验证错误"


class RepetitiveError(Exception):
    __doc__ = "数据库表字段重复错误"


class DataNotFoundError(Exception):
    __doc__ = "数据库数据未找到"


class ParamLackError(Exception):
    __doc__ = "缺少参数"


class ParamValidatedError(Exception):
    __doc__ = "自定义参数格式不正确"


class AlreadyExistsError(Exception):
    __doc__ = "资源已存在"


class CustomRaiseError(Exception):
    __doc__ = "封装自定义报错"
    """
    使用框架自动捕获异常时候使用
    use:
        raise CustomRaiseError(*e.args, int)
    params:
        *e.args: 原始的错误信息
        int: 想要的业务报错信息code
    """
class CustomException(Exception):
    """
    自定义异常类，继承自 Exception
    """
    def __init__(self, message="自定义异常"):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f"{self.message}"



"""
@Author  ：duomei
@File    ：response_utils.py
@Time    ：2025/4/8 14:23
"""

from sthg_fastapi_logs.log_util import local_trace, TraceID


def get_process_time(start_time, end_time):
    return '{}'.format(round((end_time - start_time) * 1000, 6))


def get_ip(request):
    return request.client.host


def get_header(request):
    return dict(request.headers)


def get_response_msg(response):
    if response and type(response) == dict:
        msg = response.get('msg') or response.get('message') or response.get('Message')
    else:
        msg = "-"
    return msg


def get_response_data(response):
    if response :
        msg = response
    else:
        msg = "-"
    return msg


def get_request_data(args, kwargs):
    params = {
        'args': args[1:],
        'kwargs': kwargs
    }
    if not params:
        params = '-'
    return params

# 依赖函数，用于获取所有类型的参数



def logs_set_header():
    request = local_trace.request
    request_header = get_header(request)
    guid = request_header.get('X-GUID') if request_header.get('X-GUID') else "-"
    requestId = TraceID.get_trace() if TraceID.get_trace() else "-"
    if request_header.get('X-User-ID'):
        userId = request_header.get('X-User-ID')
    elif request_header.get('user_id'):
        userId = request_header.get('user_id')
    else:
        userId = "-"
    header = {"user_ip": get_ip(request), "host": request_header['host'], 'user_id': userId}
    return f'{guid}\t {userId}\t {requestId}\t header-{header}'




import asyncio
import inspect
import logging
import os
import threading
import time
import functools
import traceback
import json
from typing import Callable, Dict, Any, Coroutine, TypeVar

from fastapi import (
    FastAPI,
    Request,
    Response, HTTPException
)

from sthg_fastapi_logs.enumerate import BusinessCode, HttpStatusCode, RESULT_SUCCESS
from sthg_fastapi_logs.log_util import create_loggers, TraceID, local_trace, get_process_funcname, get_module_line, \
    get_main_tracebak, clean_error_message
from sthg_fastapi_logs.utils.code_utils import handle_method_status, handle_business_code, get_main_traceback, \
    exception_module_func
from sthg_fastapi_logs.utils.exception_utils import CustomException
from sthg_fastapi_logs.utils.response_utils import get_process_time, get_response_msg, get_request_data, \
    get_header, get_ip, get_response_data, logs_set_header

# 声明全局变量
default_config = {
    "access": {
        "enable": True,
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.DEBUG,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    },
    "error": {
        "enable": True,
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.ERROR,
        "console_level": logging.ERROR,
        "acc_when": "W6",
        "backupCount": 0
    },
    "all_info": {
        "enable": True,
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.INFO,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    },
    "server": {
        "enable": True,
        "is_console": False,
        "is_msg": False,
        "is_request": False,
        "is_response": False,
        "logger_level": logging.DEBUG,
        "console_level": logging.INFO,
        "acc_when": "W6",
        "backupCount": 0
    }
}


def init_log_config(config: dict[str, dict] = None):
    global default_config, acc_logger, server_logger, error_logger, all_info
    if config:
        for key, value in config.items():
            for k, v in value.items():
                default_config[k] = v
    acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)


def custom_edit_log(level: str = 'info', exc: Any = None, message: Any = None):
    """
    封装的方法，用于记录异常日志和描述
    :param logger: 日志记录器
    :param description: 对异常的描述
    :param exception: 捕获到的异常对象
    """
    if 'info' == level:
        print_message = f"{message}"
        all_info.info(print_message)
    if 'error' == level:
        print_message = f"{message}: {str(exc)}"
        error_logger.error(print_message, exc_info=exc)


def get_module_func_by_router(request):
    route = request.scope.get("route")
    if route:
        endpoint = route.endpoint
        # 获取函数的定义信息
        source_lines, start_line = inspect.getsourcelines(endpoint)
        method_name = endpoint.__name__
        module_file = inspect.getmodule(endpoint).__file__
        return module_file, method_name, start_line
    else:
        # pass
        raise HTTPException(status_code=404, detail='路由未找到，请检查请求的 URL 是否正确。')


def register_log_middleware(app: FastAPI):
    # 全局变量,以便通过中间件引入,可以进行配置
    @app.middleware("http")
    async def log_middleware(request: Request, call_next):
        _request_id_key = "X-Request-Id"
        _trace_id_key = "X-Request-Id"
        _trace_id_val = request.headers.get(_trace_id_key)
        if _trace_id_val:
            TraceID.set_trace(_trace_id_val)
            TraceID.set_trace(_trace_id_val)
        else:
            TraceID.new_trace()
            _trace_id_val = TraceID.get_trace()
        local_trace.trace_id = TraceID.get_trace()
        local_trace.request = request

        async  def set_body(request: Request):
            receive_ = await  request._receive()
            async def receive():
                return receive_
            request._receive = receive

        await set_body(request)

        return await call_next(request)


def get_exception_msg(func, exc, moduler_func_line, ):
    try:
        file_path = inspect.getfile(func)
        # 获取文件所在的目录路径
        current_dir = os.path.dirname(os.path.abspath(file_path))

        moduler_func_line = moduler_func_line if moduler_func_line else get_module_line(
            traceback.format_exc(), project_path=current_dir)
        exc_type = type(exc)
        exc_value = exc
        exc_tb = exc.__traceback__
        # 调用 format_exception 函数格式化异常信息
        formatted_exception = traceback.format_exception(exc_type, exc_value, exc_tb)
        # 打印格式化后的异常信息
        main_error = "Traceback (most recent call last):\n" + get_main_tracebak(
            ''.join(formatted_exception), project_path=current_dir)
        return main_error, moduler_func_line
    except Exception as e:
        error_logger.error('日志处理异常 :{}'.format(str(e)))


def access_log(print_message:str = "默认值"):
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            if default_config['access']['enable']:
                moduler_func_line, return_desc, msg = '-', '-', '-'
                start_time = time.time()
                response = None
                access_info = {
                    'header_str': "",
                    'process_time': '-',
                    "status_code": RESULT_SUCCESS,
                    "code": BusinessCode.OK,
                    'return_desc': return_desc,
                    'msg': msg,
                    'moduler_func_line': moduler_func_line,
                    'param_input': '-',
                    'respData': '-',
                    "log_time_start": '-'
                }
                try:
                    if asyncio.iscoroutinefunction(func) or 'function' not in str(type(func)):
                        response = await func(*args, **kwargs)
                        if isinstance(response, (asyncio.Future, asyncio.Task)):
                            response = await response
                    else:
                        response = func(*args, **kwargs)
                except Exception as exc:
                    process_time = get_process_time(start_time, time.time())
                    log_time_start = time.time()
                    main_error, moduler_func_line = get_exception_msg(func, exc, moduler_func_line)  #
                    if not response:
                        response = Response(status_code=500)
                    access_info['process_time'] = process_time
                    access_info['status_code'] = 'ERROR'
                    access_info['code'] = 'InnerError'
                    access_info['return_desc'] = HttpStatusCode.get_description(response.status_code)
                    access_info['msg'] = main_error
                    access_info['moduler_func_line'] = moduler_func_line
                    access_info['respData'] = '-'
                    access_info['log_time_start'] = log_time_start
                    await get_access_log_str(**access_info)
                    raise
                try:
                    process_time = get_process_time(start_time, time.time())
                    log_time_start = time.time()
                    params = await request_params()
                    header_str = logs_set_header()
                    access_info['header_str'] = header_str
                    access_info['param_input'] = params
                    method_name = func.__name__
                    module = inspect.getmodule(func)
                    if module is not None and hasattr(module, '__file__'):
                        # 获取模块对应的文件路径
                        project_root = os.getcwd()
                        module_file = module.__file__
                        relative_path = os.path.relpath(module_file, project_root)
                        module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                        new_module_path = module_path.split('.')[-1]
                        moduler_func_line = "{}.{}".format(new_module_path, method_name)
                    if response:
                        if isinstance(response, dict):
                            new_response = response
                        else:
                            new_response = response.dict()
                        if 'code' in new_response or 'http_code' in new_response:
                            if 'code' in new_response:
                                code = new_response['code']
                            else:
                                code = new_response['http_code']
                        else:
                            raise CustomException('返回不是标准结构')
                    else:
                        raise CustomException('返回不是标准结构')
                    status_code = handle_method_status(code)
                    busiCode = handle_business_code(code)
                    if code in [0, 1]:
                        if code == 0:
                            return_desc = f"请求成功"
                        if code == 1:
                            return_desc = f"请求失败"
                    elif code in [200]:
                        return_desc = f"请求成功"
                    elif code in [400, 401, 403, 404]:
                        return_desc = HttpStatusCode.get_description(code)
                    elif code in [500]:
                        return_desc = HttpStatusCode.get_description(code)
                    else:
                        return_desc = HttpStatusCode.get_description(code)
                    access_info['process_time'] = process_time
                    access_info['status_code'] = status_code
                    access_info['code'] = busiCode
                    access_info['return_desc'] = return_desc
                    access_info['msg'] = msg
                    access_info['moduler_func_line'] = moduler_func_line
                    access_info['respData'] = response if response else '-'
                    access_info['log_time_start'] = log_time_start
                    await get_access_log_str(**access_info)
                except Exception as exc:
                    error_logger.error('日志处理异常 :{}'.format(str(exc)))
                return response
            return func(*args, **kwargs)
        return wrapper
    return decorator


"""
"time":请求时间
,"traceId":全链路Id
,"method":访问方法
,"status":http状态码
,"code":业务状态码
,"msg": 返回描述，当异常时，可以把简略堆栈放里面
,"resRT": 响应时长
,"logRT": 日志打印耗时
,"reqParams": 请求参数
,"reData": 返回参数
"""


# 记录方法执行日志
def service_log():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            if default_config['server']['enable']:
                msg = '-'
                moduler_func_line = '-'
                start_time = time.time()
                try:
                    if asyncio.iscoroutinefunction(func) or 'function' not in str(type(func)):
                        response = await func(*args, **kwargs)
                        if isinstance(response, (asyncio.Future, asyncio.Task)):
                            response = await response
                    else:
                        response = func(*args, **kwargs)
                except Exception as exc:
                    error_logger.error('日志处理异常 :{}'.format(str(exc)))
                    raise
                try:
                    process_time = get_process_time(start_time, time.time())
                    log_time_start = time.time()
                    params = get_request_data(args, kwargs)
                    method_name = func.__name__
                    module = inspect.getmodule(func)
                    if module is not None and hasattr(module, '__file__'):
                        # 获取模块对应的文件路径
                        project_root = os.getcwd()
                        module_file = module.__file__
                        relative_path = os.path.relpath(module_file, project_root)
                        module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                        new_module_path = module_path.split('.')[-1]
                        moduler_func_line = "{}.{}".format(new_module_path, method_name)
                    reData = get_response_data(response)
                    if not params: params = '-'
                    await get_service_log_str(moduler_func_line, process_time, log_time_start, RESULT_SUCCESS, "OK",
                                              "业务处理成功", msg, params, reData)
                except Exception as e:
                    error_logger.error('日志处理异常 :{}'.format(str(e)))
                return response
            return func(*args, **kwargs)
        return wrapper
    return decorator


#
def class_decorator(func):
    @functools.wraps(func)
    async def wrapper(*args, **kwargs):
        if default_config['server']['enable']:
            msg = "-"
            reData = '-'
            moduler_func_line = '-'
            params = '-'
            try:
                params = get_request_data(args, kwargs)
                method_name = func.__name__
                module = inspect.getmodule(func)
                if module is not None and hasattr(module, '__file__'):
                    # 获取模块对应的文件路径
                    project_root = os.getcwd()
                    current_dir = os.path.dirname(os.path.abspath(project_root))
                    module_file = module.__file__
                    relative_path = os.path.relpath(module_file, current_dir)
                    module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                    new_module_path = module_path.split('.')[-1]
                    moduler_func_line = "{}.{}".format(new_module_path, method_name)
            except  Exception as e:
                error_logger.error('日志处理异常 :{}'.format(str(e)))
            start_time = time.time()
            try:
                response = func(*args, **kwargs)
            except Exception as exc:
                error_logger.error('日志处理异常 :{}'.format(str(exc)))
                raise
            process_time = get_process_time(start_time, time.time())
            log_time_start = time.time()
            reData = response if response else reData
            await get_service_log_str(moduler_func_line, process_time, log_time_start, "SUCCESS", 'OK', '业务处理成功',
                                      msg, params, reData)
            return response
        return func(*args, **kwargs)

    return wrapper


def class_log(cls):
    for name, method in vars(cls).items():
        if callable(method) and name != '__init__':  # 排除__init__方法
            setattr(cls, name, class_decorator(method))
    return cls


async def get_access_log_str(**access_kwargs):
    # acc_logger, server_logger, error_logger, all_info = create_loggers(default_config)
    try:

        header_str = access_kwargs['header_str']
        msg = clean_error_message(access_kwargs['msg'])
        moduler_func_line = str(access_kwargs['moduler_func_line']).replace('...', '')
        status_code = access_kwargs['status_code']
        code = access_kwargs['code']
        return_desc = access_kwargs['return_desc']
        process_time = access_kwargs['process_time']
        param_input = access_kwargs['param_input']
        respData = access_kwargs['respData']
        log_time_start = access_kwargs['log_time_start']
        logRT = get_process_time(log_time_start, time.time())
        info_msg = f"""{moduler_func_line}\t {status_code}\t {code}\t {return_desc}\t {process_time}\t {logRT}\t {header_str}\t {param_input}\t {respData}\t {msg}"""

        acc_logger.info(str(info_msg))
    except Exception as e:
        error_logger.error('日志记录异常 :{}'.format(str(e)))


async def get_error_log(header_str, status_code, code, msg, module_line="-"):
    try:
        t1 = time.time()
        module_line = str(module_line).replace('...', '')
        logRT = get_process_time(t1, time.time())
        info_msg = f'{module_line}\t {status_code}\t {code}\t {logRT}\t {header_str}\n{msg}'
        error_logger.error(str(info_msg))
    except Exception as e:
        error_logger.error('日志记录异常 :{}'.format(str(e)))
        raise e


async def get_service_log_str(
        method, process_time, log_time_start, status_code, code, return_desc, msg, reqParam, reData):
    try:
        msg = clean_error_message(msg)
        logRT = get_process_time(log_time_start, time.time())
        service_msg = f"""{method}\t {status_code}\t {code}\t {return_desc}\t {process_time}\t {logRT}\t {reqParam}\t {reData} {msg}"""
        server_logger.debug(str(service_msg))
    except Exception as e:
        error_logger.error('service log error :{}'.format(str(e)))


async def request_params():
    try:
        request = local_trace.request
        # 查询参数
        query_params = dict(request.query_params)

        content_type = request.headers.get('content-type') if request.headers.get('content-type') else ''
        # 表单数据
        form_data = {}
        # 不可识别数据
        others = ''
        # 请求体数据
        body_data = {}
        if content_type and 'application/json' in content_type:
            body_data = await asyncio.wait_for(request.body(), timeout=5)
            body_data = json.dumps(body_data.decode())
        elif content_type and 'multipart/form-data' in content_type:
            if 'multipart/form-data; boundary=' in content_type:
                form = await asyncio.wait_for(request.form(), timeout=5)
                has_file = False
                for field_name, value in form.items():
                    if hasattr(field_name, 'filename'):
                        has_file = True
                        break
                if has_file:
                    form_data["file_upload"] = 1
                else:
                    form = await asyncio.wait_for(request.form(), timeout=5)
                    form_data = {key: value for key, value in form.items()}
            else:
                form = await asyncio.wait_for(request.form(), timeout=5)
                form_data = {key: value for key, value in form.items()}
        elif content_type and 'application/x-www-form-urlencoded' in content_type:
            form = await asyncio.wait_for(request.form(), timeout=5)
            form_data = {key: value for key, value in form.items()}
        else:
            others = '-'
        params_input = {}
        if query_params:
            params_input['params'] = query_params
        if form_data:
            params_input['form_data'] = form_data
        if body_data:
            params_input['body'] = body_data
        if others:
            params_input['others'] = others
        return params_input
    except Exception as e:
        error_logger.error('日志处理异常 :{}'.format(str(e)))




import asyncio
import inspect
import logging
import uuid
import os
from datetime import datetime
import threading
from contextvars import ContextVar
from logging.handlers import TimedRotatingFileHandler

from fastapi.routing import APIRoute

# 自定义trace_filter属性名
TRACE_FILTER_ATTR = "trace_filter"
# 当前线程的local_trace, 需要添加全局trace_id, 使用示例：trace.trace_id
local_trace = threading.local()
formatter = logging.Formatter(
    '%(asctime)s \t %(trace_id)s \t %(name)s \t %(levelname)s \t %(message)s')
_trace_id: ContextVar[str] = ContextVar('x_trace_id', default="-")
_x_request_id: ContextVar[str] = ContextVar('_x_request_id', default="-")

# 自定义日志格式器，用于格式化包含根方法路径的日志消息
class CustomFormatter(logging.Formatter):
    def format(self, record):
        # 定义自定义格式字符串，包含根方法路径
        format_string = '%(asctime)s - %(name)s - %(levelname)s - %(root_method_path)s - %(message)s'
        return super().format(record, format_string)


class TraceID:
    @staticmethod
    def set(req_id: str) -> ContextVar[str]:
        """设置请求ID，外部需要的时候，可以调用该方法设置
        Returns:
            ContextVar[str]: _description_
        """
        if req_id:
            _x_request_id.set(req_id)
        return _x_request_id

    @staticmethod
    def set_trace(trace_id: str) -> ContextVar[str]:
        """设置trace_id
        Returns:
            ContextVar[str]: _description_
        """
        if trace_id:
            _trace_id.set(trace_id)
        return _trace_id

    @staticmethod
    def new_trace():
        trace_id = uuid.uuid4().hex
        _trace_id.set(trace_id)

    @staticmethod
    def get_trace() -> str:

        """获取trace_id
        Returns:
            str: _description_
        """
        return _trace_id.get()


class TraceFilter(logging.Filter):
    """
    通过在record中添加trace_id, 实现调用跟踪和日志打印的分离
    """

    def __init__(self, name=""):
        """
        init
        @param name: filter name
        """
        super().__init__(name)

    def filter(self, record):
        """
        重写filter方法
        @param record: record
        @return:
        """
        # trace_id = local_trace.trace_id if hasattr(local_trace, 'trace_id') else uuid.uuid1()
        record.trace_id = _trace_id.get()
        return True

class TraceLogger:
    _loggers = {}
    @staticmethod
    def get_log_file_path(logger_name):
        log_dir_name = 'logs'
        current_working_directory = os.getcwd()
        log_dir = os.path.join(current_working_directory, log_dir_name)
        if not os.path.exists(log_dir):
            os.mkdir(log_dir)
        log_file_name = "{}_log.{}.log".format( logger_name, str(datetime.now().strftime('%Y-%m-%d')))
        log_file_path = os.path.join(log_dir, log_file_name)
        return log_file_path
    @staticmethod
    def get_logger_func(logger_name,config):
        if logger_name in TraceLogger._loggers:
            return TraceLogger._loggers[logger_name]
        # 创建一个名为'access'的日志记录器实例
        access_logger = logging.getLogger(logger_name)
        # 设置日志记录器的最低捕获级别
        access_logger.setLevel(config['logger_level'])
        # 避免重复添加过滤器
        if not any(isinstance(f, TraceFilter) for f in access_logger.filters):
            # 添加日志跟踪过滤器
            trace_filter = TraceFilter()
            access_logger.addFilter(trace_filter)
        access_filename = TraceLogger.get_log_file_path(logger_name)
        if access_filename:
            # 避免重复添加文件处理器
            if not any(isinstance(h, TimedRotatingFileHandler) for h in access_logger.handlers):
                # 创建一个按时间滚动的文件处理器
                file_handler = TimedRotatingFileHandler(
                    access_filename,
                    when=config['acc_when'],
                    backupCount=config["backupCount"],
                    encoding='utf-8'
                )
                # 设置文件处理器的格式化器
                file_handler.setFormatter(formatter)
                # 将文件处理器添加到日志记录器中
                access_logger.addHandler(file_handler)
        if config['is_console']:
            # 避免重复添加控制台处理器
            if not any(isinstance(h, logging.StreamHandler) for h in access_logger.handlers):
                # 再创建一个handler，用于将日志输出到控制台
                console_handler = logging.StreamHandler()
                console_handler.setLevel(config['console_level'])  # 设置控制台handler的日志级别
                console_handler.setFormatter(formatter)
                access_logger.addHandler(console_handler)
        TraceLogger._loggers[logger_name] = access_logger
        return access_logger

# 定义日志记录器
def create_loggers(config):
    acc_config = config['access']
    server_config = config['server']
    err_config = config['error']
    info_config = config['all_info']
    acc_logger, server_logger, error_logger, all_info = None, None, None, None
    if acc_config.get('enable'):
        acc_logger = TraceLogger.get_logger_func("access", config=acc_config)
    if server_config.get('enable'):
        server_logger = TraceLogger.get_logger_func("server", config=server_config)
    if err_config.get('enable'):
        error_logger = TraceLogger.get_logger_func("error",config=err_config)
    if info_config.get('enable'):
        all_info = TraceLogger.get_logger_func("info", config=info_config)
    return acc_logger, server_logger, error_logger,all_info

def get_module_line(e: any,project_path=None):

    module_path = '-'
    stack_lines = e.strip().split('\n')
    project_root = project_path if  project_path else os.getcwd()  # 假设项目根目录为当前工作目录
    for line in stack_lines:
        if line.startswith('  File'):
            parts = line.split('"')
            if len(parts) > 1:
                file_path = parts[1]
                if file_path.startswith(project_root):
                    relative_path = os.path.relpath(file_path, project_root)
                    module_path = relative_path.replace(os.sep, '.')[:-3]  # 去除 .py 扩展名
                    # 从 parts 中进一步提取方法名
                    function_part = parts[-1].strip()
                    if function_part.startswith(', line'):
                        sub_parts = function_part.split(',')
                        if len(sub_parts) > 2 and sub_parts[2].strip().startswith('in '):
                            function_name = sub_parts[2].strip().split(' ')[-1]
    return f"{module_path}.{function_name}"


def get_process_funcname(app, request):
    moduler_func_line = None
    func_module, func_name, func_line = None, None, None
    try:
        for route in app.routes:
            if route.path.startswith("/docs") or route.path.startswith("/redoc") or route.path.startswith("/openapi.json"):
                continue
            match, matched_path_params = route.matches(request.scope)
            if match:
                methods = set()
                if isinstance(route, APIRoute):
                    methods = route.methods

                if request.method.lower() in [method.lower() for method in methods]:
                #     print(f"匹配到的路由路径: {route.path}")  # 调试信息
                    # func_name = route.endpoint.__name__
                    if request.scope.get('path') == route.path:
                        endpoint = route.endpoint
                        func_name = endpoint.__name__ if hasattr(endpoint, '__name__') else str(endpoint)
                    else:
                        continue
                    try:
                        source_lines, start_line = inspect.getsourcelines(route.endpoint)
                        func_line = start_line
                        module = inspect.getmodule(route.endpoint)
                        if module:
                            full_module_name = module.__name__
                            if '.' in full_module_name:
                                func_module = full_module_name.split('.')[-1]
                            else:
                                func_module = full_module_name
                        else:
                            func_module = "无法获取模块"
                    except (TypeError, OSError) as e:
                        pass
                    break
    except Exception as e:
        pass
    if func_module and func_name and func_line:
        moduler_func_line = "{}.{}".format(func_module, func_name)
    return moduler_func_line


def get_main_tracebak(stack_trace,project_path=None):
    current_dir = os.path.dirname(os.path.abspath(__file__))
    # 定义你的项目路径
    project_path = project_path if project_path else os.getcwd()

    # 分割堆栈跟踪信息为行
    lines = stack_trace.splitlines()

    # 提取主要堆栈内容
    main_stack = []
    i = 0
    while i < len(lines):
        line = lines[i]
        if project_path in line:
            main_stack.append(line)
            if i + 1 < len(lines):
                main_stack.append(lines[i + 1])
            i += 2  # 跳过下一行，因为已经添加了
        else:
            i += 1
    # 打印主要堆栈内容
    main_tracebak = ''.join(main_stack).replace('\t','').replace('\n','') + lines[-1] if len(lines) > 0 else ''
    return main_tracebak

def clean_error_message(error_message):
    # 将换行符替换为空格
    error_message = error_message.replace('\n', ' ')
    # 将制表符替换为空格
    error_message = error_message.replace('\t', ' ')
    # 将两个及以上的连续空格替换为一个空格
    while '  ' in error_message:
        error_message = error_message.replace('  ', ' ')
    return error_message



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Author  ：LiShun
@File    ：exception.py
@Time    ：2022/7/12 18:56
@Desc    ：
"""
# __all__ = [
#     'BaseHTTPException', 'register_exception'
# ]

import os
import time
import traceback
from typing import Any, Optional, Dict

from fastapi import FastAPI
from fastapi import Request
from fastapi.exceptions import HTTPException, RequestValidationError
from fastapi.responses import JSONResponse, Response
from pydantic import ValidationError

from .base_code.code import CODE
from sthg_fastapi_logs.utils.code_utils import handle_method_status, handle_business_code, exception_module_func
from .enumerate import  HttpStatusCode
from .log_util import get_main_tracebak
from .log_wrapper import get_error_log, get_service_log_str, get_process_time
from .utils.response_utils import logs_set_header


def json_response(data=None, message=None, code=200, status_code=200) -> Response:
    code_dict = CODE.get(code) or CODE[400]
    return JSONResponse(
        status_code=status_code,
        content={
            "code": code_dict["code"],
            "message": message or code_dict["zh-cn"],
            "data": data
        }
    )


class BaseHTTPException(HTTPException):
    EXC_STATUS_CODE = 500
    EXC_CODE = 1000
    EXC_MESSAGE = None

    def __init__(
            self,
            message: Any = None,
            status_code: int = 500,
            code: int = 40000,
            headers: Optional[Dict[str, Any]] = None
    ) -> None:
        self.message = message or self.EXC_MESSAGE
        self.status_code = status_code or self.EXC_STATUS_CODE
        self.code = code or self.EXC_CODE
        self.headers = headers

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r}, message={self.message!r})"


def register_log_exception(app: FastAPI):
    """
    捕获FastApi异常
    :param app:
    :return:
    """
    start_time = time.time()
    async def log_and_response(request, exc, default_code=500, status_code=500):
        msg = traceback.format_exc()
        current_file_dir = os.path.dirname(os.path.abspath(__file__))
        # project_folder = os.path.dirname(current_file_dir)
        module_line, main_error, method_name, project_root,line = exception_module_func(request, exc)
        new_status_code = status_code
        if status_code:
            status_code = handle_method_status(default_code)
        code = handle_business_code(default_code)
        header_str = logs_set_header()
        await get_error_log(header_str=header_str, status_code=status_code, code=code, msg=msg, module_line=module_line)


        server_msg = get_main_tracebak(traceback.format_exc(), project_path=project_root)
        if "File" not in server_msg:
            server_msg = "File '{}', line {} in {} {}".format(project_root,line,method_name,server_msg)
        server_msg = "Traceback (most recent call last):\n" + server_msg
        module_line = module_line.replace('...','.')
        process_time = get_process_time(start_time, time.time())

        await get_service_log_str(module_line, process_time=process_time, log_time_start=time.time(),status_code=status_code, code=code,
                            return_desc=HttpStatusCode.get_description(new_status_code), msg=server_msg,
                            reqParam="-",reData='-')
        message = str(exc) if not isinstance(exc, AssertionError) else ''.join(exc.args)
        if isinstance(exc, RequestValidationError):
            message = f'{exc}'.replace('\n', ' ').replace('   ', ' ')
        if isinstance(exc, HTTPException):
            message = str(exc.detail)
        if isinstance(exc, BaseHTTPException):
            message = str(exc.message)
        return json_response(code=default_code, message=message, status_code=new_status_code)

    ####################
    @app.exception_handler(Exception)
    async def exception_handle(request: Request, exc: Exception):
        # 处理所有未捕获的异常
        if "status_code" in exc.__dict__:
            new_status_code = exc.__dict__.get("status_code")
            new_code = exc.__dict__.get("status_code")
        else:
            new_status_code = 500
            new_code = 500
        return await log_and_response(request, exc, default_code=new_status_code, status_code=new_code)

    @app.exception_handler(HTTPException)
    async def http_exception_handle(request: Request, exc: HTTPException):
        # 处理HTTP异常
        if "status_code" in exc.__dict__:
            new_code = exc.__dict__.get("status_code")
        else:
            new_code = 500
        return await log_and_response(request, exc, default_code=new_code, status_code=new_code)

    @app.exception_handler(BaseHTTPException)
    async def base_http_exception_handle(request: Request, exc: BaseHTTPException):
        # 处理自定义的BaseHTTPException异常
        if "status_code" in exc.__dict__:
            new_code = exc.__dict__.get("status_code")
        else:
            new_code = 404
        return await log_and_response(request, exc, default_code=new_code, status_code=new_code)

    @app.exception_handler(AssertionError)
    async def assert_exception_handle(request: Request, exc: AssertionError):
        # 处理断言错误
        if "status_code" in exc.__dict__:
            new_code = exc.__dict__.get("status_code")
        else:
            new_code = 400
        return await log_and_response(request, exc, default_code=new_code, status_code=new_code)

    @app.exception_handler(ValidationError)
    async def validation_exception_handle(request: Request, exc: ValidationError):
        # 处理验证错误
        if "status_code" in exc.__dict__:
            new_code = exc.__dict__.get("status_code")
        else:
            new_code = 400
        return await log_and_response(request, exc, default_code=new_code, status_code=new_code)

    @app.exception_handler(RequestValidationError)
    async def request_exception_handler(request: Request, exc: RequestValidationError):
        # 处理请求验证错误
        if "status_code" in exc.__dict__:
            new_code = exc.__dict__.get("status_code")
        else:
            new_code = 400
        return await log_and_response(request, exc, default_code=new_code, status_code=new_code)
